/* 
 * File:   gui.h
 * Author: Tim Braren
 *         Dhruv Gaba
 *         Shaurya Luthra
 *
 * Created on November 7, 2017, 5:49 PM
 */
#include "neopixel.h"

#ifndef GUI_H
#define	GUI_H

// Number of data entries stored in for each sensor for graphing
#define DATAPOINTS 270
#define NUM_SENSORS 7

int selected_sensor = 0;
float entered = 0;

// For settings screen
int x_offset = 1;
int y_offset = 40;
int y_trig_h;
int y_trig_l;

int param = 0;
#define SENSOR 0
#define COLOR 1
#define TRIG_HIGH 2
#define TRIG_LOW 3

#define STATIC 1
#define NO_STATIC 0

// Flag for encoding whether or not to draw static components on the screen
int draw = STATIC;

// === GUI Variables ===========================================================
// Character buffer for displaying strings on the TFT.
char buffer[120];

unsigned short colors[7] = 
{
  ILI9340_BLUE,
  ILI9340_GREEN,
  ILI9340_CYAN,
  ILI9340_MAGENTA,
  ILI9340_YELLOW,
  ILI9340_WHITE,
  ILI9340_RED
 };

int color_idx = 0;

// music *
// graphs #
// settings 0
#define MAIN_SCREEN 1
#define SETTINGS_SCREEN 0
#define MUSIC_SCREEN 10
#define GRAPH_SCREEN 11 

typedef struct 
{
  char *name;
  int data[DATAPOINTS];
  float max;
  float min;
  float trig_high;
  float trig_low;
  float value;
  int color;
} sensor;

sensor sensors[NUM_SENSORS];

// properly add data to an end of an array
void add_data(sensor* s)
{
  int i;
  for(i=0; i<DATAPOINTS - 1; i++)
  {
	s->data[i] = s->data[i+1];
  }
  s->data[DATAPOINTS - 1] = 210-(int)((((s->value-s->min)/(s->max-s->min)))*160.0);
}

// draw continuous line graphs based on last 320 sensor data points
void draw_graph(sensor* s)
{    
  tft_fillRoundRect(50, 51, 270, 160, 1, ILI9340_BLACK);
  tft_drawLine(50, 50, 320, 50, ILI9340_WHITE);
  tft_drawLine(50, y_trig_h, 320, y_trig_h, ILI9340_RED);
  tft_drawLine(50, y_trig_l, 320, y_trig_l, ILI9340_RED);
  tft_drawLine(50, 211, 320, 211, ILI9340_WHITE);
  int i;
  for(i = 0; i< DATAPOINTS - 2; i++)
  {
	if(s->data[i] != NULL && s->data[i+1] != NULL)
    {
	  tft_drawLine(i+50, s->data[i], i+1+50, s->data[i+1], s->color);
	}
  }
}

void sensors_setup(void) 
{
    sensors[0].name = "Light [Lux]";
    sensors[0].max = 6000;
    sensors[0].min = 0;
    sensors[0].trig_high = 3000;
    sensors[0].trig_low = 0;
    sensors[0].color = colors[0];
    
    sensors[1].name = "Temperature [F]";
    sensors[1].max = 90;
    sensors[1].min = 60;
    sensors[1].trig_high = 80;
    sensors[1].trig_low = 65;
    sensors[1].color = colors[1];
    
    sensors[2].name = "Humidity [%]";
    sensors[2].max = 80;
    sensors[2].min = 5;
    sensors[2].trig_high = 50;
    sensors[2].trig_low = 10;
    sensors[2].color = colors[2];
    
    sensors[3].name = "Mic Power";
    sensors[3].max = 8000;
    sensors[3].min = 0;
    sensors[3].trig_high = 3500;
    sensors[3].trig_low = 0;
    sensors[3].color = colors[3];

    sensors[4].name = "X-Accel [g]";
    sensors[4].max = 0.0;
    sensors[4].min = -2.0;
    sensors[4].trig_high = -0.6;
    sensors[4].trig_low = -1.40;
    sensors[4].color = colors[4];
    
    sensors[5].name = "Y-Accel [g]";
    sensors[5].max = 1.1;
    sensors[5].min = -1.1;
    sensors[5].trig_high = 0.5;
    sensors[5].trig_low = -0.5;
    sensors[5].color = colors[5];
    
    sensors[6].name = "Z-Accel [g]";
    sensors[6].max = 1.1;
    sensors[6].min = -1.1;
    sensors[6].trig_high = 0.3;
    sensors[6].trig_low = -0.3;
    sensors[6].color = colors[6];
}

void main_screen(void) 
{
  if (draw == STATIC)
  {
    // Black out previous numbers
    tft_fillRoundRect(0, 0, 320, 240, 1, ILI9340_BLACK);
    tft_setTextColor(ILI9340_WHITE);
    tft_setTextSize(2);
    tft_setCursor(0, 225);
    sprintf(buffer,"<-Music(*)");
    tft_writeString(buffer);

    tft_setCursor(90, 1);
    sprintf(buffer,"^Settings^(0)");
    tft_writeString(buffer); 

    tft_setCursor(190, 225);
    sprintf(buffer,"Graphs(#)->");
    tft_writeString(buffer);
    draw = NO_STATIC;
  }
  
  int color = rand() & 0xFFFF;
  tft_setTextColor(color);
  tft_setTextSize(4);
  tft_setCursor(110,65);
  sprintf(buffer,"Smart");
  tft_writeString(buffer);    
  
  tft_setCursor(85,115);
  sprintf(buffer,"Station");
  tft_writeString(buffer); 
}

void settings_screen(void) 
{
  // Settings Menu
  tft_setTextColor(ILI9340_WHITE);
  tft_setTextSize(2);
  
  if (draw == STATIC)
  {
    // Black out previous numbers
    tft_fillRoundRect(0, 0, 320, 240, 1, ILI9340_BLACK);

    tft_setCursor(x_offset, y_offset);
    tft_setTextColor(ILI9340_WHITE);
    sprintf(buffer,"Sensor: %s", sensors[selected_sensor].name);
    tft_writeString(buffer);


    tft_setCursor(x_offset, 2*y_offset);
    sprintf(buffer,"Color : ");
    tft_writeString(buffer);

    tft_fillRoundRect(94, 2*y_offset-3, 19, 19, 1, sensors[selected_sensor].color);

    tft_setCursor(x_offset, 3*y_offset);
    sprintf(buffer,"Trig H: %.2f", sensors[selected_sensor].trig_high);
    tft_writeString(buffer);

    tft_setCursor(x_offset, 4*y_offset);
    sprintf(buffer,"Trig L: %.2f", sensors[selected_sensor].trig_low);
    tft_writeString(buffer);

    tft_setCursor(1, 225);
    sprintf(buffer,"<-Back(*)");
    tft_writeString(buffer);

    tft_setCursor(118, 225);
    sprintf(buffer,"Toggle(0)");
    tft_writeString(buffer); 

    tft_setCursor(238, 225);
    sprintf(buffer,"Next(#)");
    tft_writeString(buffer);

    tft_fillCircle(311, (param + 1)*y_offset + 6, 8, ILI9340_GREEN);    
    
    draw = NO_STATIC;
  }
  
  tft_setCursor(75,1);
  int color = rand() & 0xFFFF;
  tft_setTextColor(color);
  sprintf(buffer,"Sensor Settings ");
  tft_writeString(buffer);
   
}

void music_screen(void) 
{
  int scale = 50;
  int size = 30;
  int reverse_x = 109;
  int y = 90;
  int y2 = y + size*2.1;
    
  int playpause_x = reverse_x + scale;
  int forward_x = playpause_x + scale + size*(1.0/2.0);
    
  int volumedown_x = reverse_x ;
  int volumeup_x = forward_x - (size/4.0);
  
  if (draw == STATIC)
  {
    // Black out previous numbers
    tft_fillRoundRect(0, 0, 320, 240, 1, ILI9340_BLACK);
    tft_setTextSize(4);

    // Next track graphic
    tft_fillTriangle(forward_x, y,
                     forward_x, y + size,
                     forward_x + (size/2.0), y + (size/2.0), ILI9340_WHITE);

    tft_fillTriangle(forward_x + (size/2.0), y,
                     forward_x + (size/2.0), y + size,
                     forward_x + (size), y + (size/2.0), ILI9340_WHITE);

    // Previous track graphic
    tft_fillTriangle(reverse_x, y,
                     reverse_x , y + size,
                     reverse_x - (size/2.0), y + (size/2.0), ILI9340_WHITE);

    tft_fillTriangle(reverse_x - (size/2.0), y,
                     reverse_x - (size/2.0), y + size,
                     reverse_x - (size), y + (size/2.0), ILI9340_WHITE);

    // Play/pause graphic
    tft_fillRect(playpause_x, y, size/4.0, size, ILI9340_WHITE);

    tft_fillTriangle(playpause_x + size/2.0, y,
                     playpause_x + size/2.0, y + size,
                     playpause_x + size, y + (size/2.0), ILI9340_WHITE);

    tft_setTextColor(ILI9340_WHITE); 
    tft_setTextSize(2);
    tft_setCursor(reverse_x - size, y + size + 2);
    sprintf(buffer,"~1~   ~2~   ~3~ ");
    tft_writeString(buffer); 

    // Volume down
    tft_fillTriangle(volumedown_x, y2,
                     volumedown_x, y2 + size,
                     volumedown_x - size, y2 + (size/2.0), ILI9340_WHITE);
    tft_fillRect(volumedown_x - size, y2 + size*(1.0/4.0) + 1, 
                 size/3.0, size/2.0, ILI9340_WHITE);
    tft_fillRect(volumedown_x + size*(1.0/4.0), y2 + size*(3.0/8.0), 
                 size, size*(1.0/4.0), ILI9340_WHITE);

    // Volume up
    tft_fillTriangle(volumeup_x, y2,
                     volumeup_x, y2 + size,
                     volumeup_x - size, y2 + (size/2.0), ILI9340_WHITE);
    tft_fillRect(volumeup_x - size, y2 + size*(1.0/4.0) + 1, 
                 size/3.0, size/2.0, ILI9340_WHITE);
    tft_fillRect(volumeup_x + size*(5.0/8.0), y2, 
                 size*(1.0/4.0), size, ILI9340_WHITE);
    tft_fillRect(volumeup_x + size*(1.0/4.0), y2 + size*(3.0/8.0), 
                     size, size*(1.0/4.0), ILI9340_WHITE);

    tft_setTextColor(ILI9340_WHITE); 
    tft_setTextSize(2);
    tft_setCursor(reverse_x - size, y2 + size + 2);
    sprintf(buffer," ~4~      ~5~");
    tft_writeString(buffer);

    tft_setCursor(1, 225);
    sprintf(buffer,"<-Back(*) ");
    tft_writeString(buffer);
    
    draw = NO_STATIC;
  }
  tft_setTextSize(4);
  tft_setCursor(107,25);
  int color = rand() & 0xFFFF;
  tft_setTextColor(color);
  sprintf(buffer,"Music");
  tft_writeString(buffer);
}

// Check to see if data is outside of specified bounds for that sensor
void check_data(sensor* s)
{
  // boolean for checking if data is outside a defined range
  int check = 1;
  int i;
  for(i = (DATAPOINTS - 5); i < DATAPOINTS; i++)
  {
    if(s->data[i] != NULL && (s->data[i] < y_trig_h || s->data[i] > y_trig_l))
    {
      check = 0;
    }
  }
  if(check == 0)
  {
    tft_fillCircle(310, 28, 8, ILI9340_RED);
    red();
    display();
  }
  else
  {
    tft_fillCircle(310, 28, 8, ILI9340_GREEN);
    green();
    display();
  }
}

void graph_screen(void) 
{
  tft_setTextColor(ILI9340_YELLOW); 
  tft_setTextSize(2);
    
  if (draw == STATIC)
  {
    // Black out previous numbers
    tft_fillRoundRect(0, 0, 320, 240, 1, ILI9340_BLACK);

    tft_setCursor(1,1);
    tft_setTextColor(ILI9340_WHITE); 
    tft_setTextSize(2);
    sprintf(buffer,"Sensor: %s", (sensors[selected_sensor].name)); 
    tft_writeString(buffer);

    tft_setCursor(220,20);
    sprintf(buffer,"Status: ");
    tft_writeString(buffer);

    tft_setCursor(1,20);
    sprintf(buffer,"Value: %.2f", sensors[selected_sensor].value); 
    tft_writeString(buffer);

    tft_setCursor(1,225);
    sprintf(buffer,"<-Back(*) ");
    tft_writeString(buffer);

    tft_setTextColor(ILI9340_WHITE); 
    tft_setTextSize(1);
    tft_setCursor(1,48);
    sprintf(buffer,"%.1f", (sensors[selected_sensor].max)); 
    tft_writeString(buffer);
    tft_drawLine(50, 50, 320, 50, ILI9340_WHITE);
    
    y_trig_h = 210 - (int)((((sensors[selected_sensor].trig_high - sensors[selected_sensor].min)/(sensors[selected_sensor].max - sensors[selected_sensor].min)))*160.0);
    tft_drawLine(50, y_trig_h, 320, y_trig_h, ILI9340_RED);
    tft_setCursor(1,y_trig_h-2);
    sprintf(buffer,"%.1f", (sensors[selected_sensor].trig_high)); 
    tft_writeString(buffer);
    
    y_trig_l = 210 - (int)((((sensors[selected_sensor].trig_low - sensors[selected_sensor].min)/(sensors[selected_sensor].max - sensors[selected_sensor].min)))*160.0);
    tft_drawLine(50, y_trig_l, 320, y_trig_l, ILI9340_RED);
    tft_setCursor(1,y_trig_l-2);
    sprintf(buffer,"%.1f", (sensors[selected_sensor].trig_low)); 
    tft_writeString(buffer);
    
    tft_setCursor(1,208);
    sprintf(buffer,"%.1f", (sensors[selected_sensor].min)); 
    tft_writeString(buffer);
    tft_drawLine(50, 211, 320, 211, ILI9340_WHITE);
        
    draw = NO_STATIC;
  }
  
  // Black out previous numbers
  tft_fillRoundRect(78, 18, 120, 25, 1, ILI9340_BLACK);

  tft_setCursor(80,20);
  tft_setTextColor(ILI9340_WHITE);
  sprintf(buffer,"%.2f", sensors[selected_sensor].value); 
  tft_writeString(buffer);
    
  add_data(&sensors[selected_sensor]);
  draw_graph(&sensors[selected_sensor]);
  check_data(&sensors[selected_sensor]);
}

#endif	/* GUI_H */

